<?php

namespace Modules\Leaderboard\Http\Controllers;

use App\Models\OfferLog;
use App\Models\Transaction;
use Illuminate\Routing\Controller;
use Illuminate\Contracts\Support\Renderable;
use Modules\Leaderboard\Entities\Leaderboard;
use Modules\Leaderboard\Entities\LeaderboardLog;

class LeaderboardController extends Controller
{
    /**
     * Display the leaderboard.
     *
     * @param string $type
     * @return Renderable
     */
    public function index($type = 'monthly')
    {
        abort_if(!in_array($type, ['daily', 'monthly']), 404);

       
        $users = $type === 'daily' ? $this->getUsersDaily() : $this->getUsersMonthly();
       

        $userPoints = $this->getAuthUserPoints($type);

        $levels = Leaderboard::where('type', $type)
            ->orderByDesc('reward')
            ->get();

        $topUsers = !empty($users) ? $users->take($levels->count()) : [];

        return view('leaderboard::user.index', compact('users', 'levels', 'userPoints', 'type', 'topUsers'));
    }

    /**
     * Get users for the daily leaderboard.
     */
    protected function getUsersDaily()
    {
        $lastcron = $this->getLastCron('daily');
        
        $log =  OfferLog::with('users:id,username,balance');
        
        if($lastcron){
            $log = $log->where('created_at', '>=', $lastcron);    
        }else{
            $log = $log->where('created_at', '>=', today()->subDay());
        }
        
        
        $log = $log->selectRaw('user_id, SUM(amount) as total_earning')
        ->groupBy('user_id')
        ->orderByDesc('total_earning')
        ->limit(15)
        ->get();
        
        return $log;
    }

    /**
     * Get users for the monthly leaderboard.
     */
    protected function getUsersMonthly()
    {
        $lastcron = $this->getLastCron('monthly');
        
        $log = OfferLog::with('users:id,username,balance');
        
        if($lastcron){
            $log =  $log->where('created_at', '>=', $lastcron);
        }else{
            
            $log =  $log->where('created_at', '>=', today()->subMonth());
        }
            
            
        $log= $log->selectRaw('user_id, SUM(amount) as total_earning')
            ->groupBy('user_id')
            ->orderByDesc('total_earning')
            ->limit(15)
            ->get();
            
            
        return $log;
    }

    public function getAuthUserPoints($type = 'daily')
    {
        
        $lastcron = $this->getLastCron($type);
        
        if($type == 'daily'){
            
          $log =  OfferLog::where('user_id', auth()->id());
            
            if($lastcron){
                $log = $log->where('created_at', '>=', $lastcron);
            }else{
                $log = $log->where('created_at', '>=', today()->subDay());
            }
            
            $log =  $log->sum('amount');
        }else{
            
           $log =  OfferLog::where('user_id', auth()->id());
            
            if($lastcron){
                $log = $log->where('created_at', '>=', $lastcron);
            }else{
                $log = $log->where('created_at', '>=', today()->subMonth());
            }
            
            $log =  $log->sum('amount');
        }
        
        return $log;
    }
    
    
    protected function getLastCron($type = 'daily')
    {
        $time = $type == 'daily' ? today()->subDay() : today()->subMonth();
        
        $query = LeaderboardLog::where('created_at', '>=' , $time)
                                ->orderByDesc('created_at')
                                ->first();
        
        if(!is_null($query)){
            return $query->created_at;
        }
        
        return false;
    }

    /**
     * Perform cron job to pay users based on leaderboard type.
     *
     * @param string $type The leaderboard type ('daily' or 'monthly')
     * @return string
     */
    public function cronjob($type = 'daily')
    {
        // Validate leaderboard type
        abort_if(!in_array($type, ['daily', 'monthly']), 404);

        //check if cronjob is completed before
        if($type == 'daily'){
            $log = LeaderboardLog::where('created_at', '>=' ,today()->subDay())->count();
        }else{
            $log = LeaderboardLog::where('created_at', '>=' ,today()->subMonth())->count();
        }
        

        if($log > 0){
            return 'Cronjob completed!';
        }

        // Get users based on leaderboard type
        $users = $type === 'daily' ? $this->getUsersDaily() : $this->getUsersMonthly();

        // Get leaderboard levels for the given type
        $levels = Leaderboard::where('type', $type)
            ->orderByDesc('reward')
            ->get();

        // Get the top users based on the number of levels
        $topUsers = $users->take($levels->count());

        // Process payouts for each level and user
        foreach ($levels as $index => $level) {

            if (isset($topUsers[$index])) {

                $user = $topUsers[$index];

                // Increment user's balance and create a leaderboard log
                $user->users->increment('balance', $level->reward);

                $leaderLog = LeaderboardLog::create([
                    'user_id' => $user->users->id,
                    'reward' => $level->reward,
                    'type' => $type,
                ]);

                //save transactions
                Transaction::store([
                    'user' => $user->users,
                    'amount' => getAmount($level->reward),
                    'from' => 'LEADERBOARD',
                    'source_id' => $leaderLog->id,
                    'details' => 'Leaderboard Earnings',
                ]);
            }
        }

        return 'Cronjob completed!';
    }


    public function history($type = 'daily')
    {
        abort_if(!in_array($type, ['daily', 'monthly']), 404);

        return view('leaderboard::user.history', [
            'logs' => LeaderboardLog::where('type', $type)
                ->paginate(),
            'type' => $type
        ]);
    }
}
