<?php

namespace App\Http\Controllers\Auth;

use App\Models\User;
use App\Http\Helper\Captcha;
use Illuminate\Http\Request;
use App\Models\User\UserProfile;
use App\Models\AdminNotification;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Laravel\Socialite\Facades\Socialite;
use App\Http\Traits\AuthTrait;
use Illuminate\Foundation\Auth\AuthenticatesUsers;


class LoginController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Login Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles authenticating users for the application and
    | redirecting them to your home screen. The controller uses a trait
    | to conveniently provide its functionality to your applications.
    |
    */

    use AuthenticatesUsers, AuthTrait;

    /**
     * Where to redirect users after login.
     *
     * @var string
     */

    protected $username;

    /**
     * Create a new controller instance.
     *
     * @return void
     */


    public function __construct()
    {
        $this->middleware('guest')->except('logout');
        $this->username = $this->findUsername();
    }

    public function findUsername()
    {
        $login = request()->input('username');

        $fieldType = filter_var($login, FILTER_VALIDATE_EMAIL) ? 'email' : 'username';
        request()->merge([$fieldType => $login]);
        return $fieldType;
    }

    public function showLoginForm()
    {

        return view( SETTING['site_theme'] . 'auth.login', [
            'enabledCaptcha' => SETTING['en_cap_login'],
            'captcha' => (new Captcha())->create(),
        ]);
    }

    public function login(Request $request)
    {
        if (SETTING['en_cap_login']) {
            if (!(new Captcha())->check($request->toArray()))
            {
                return redirect()->route('user.login')
                    ->withNotify([['error', 'Invalid captcha']]);
            }
        }

        $this->validateLogin($request);

        if ($this->hasTooManyLoginAttempts($request)) {
            $this->fireLockoutEvent($request);

            return $this->sendLockoutResponse($request);
        }
    
        
        $username = $request->input('username');
        $password = $request->input('password');
        
        if (filter_var($username, FILTER_VALIDATE_EMAIL))
        {
            // email login
            if(Auth::attempt(['email'=>$username, 'password'=>$password], $request->remember))
            {
                $this->loginLog(auth()->user());
                return $this->sendLoginResponse($request);
            }
            
        }
        else{
            // username login
            $user = User::where('username', $username)->first();
            if($user){
                if(Hash::check(trim($password), $user->password)){
                    Auth::login($user);
                    $this->loginLog($user);
                    return $this->sendLoginResponse($request);
                }
            }
        }
        
        
        $this->incrementLoginAttempts($request);


        return $this->sendFailedLoginResponse($request);
    }

    protected function validateLogin(Request $request)
    {
        $validation_rule = [
            $this->username() => 'required|string',
            'password' => 'required|string',
        ];
        
        $request->validate($validation_rule);
    }

    public function username()
    {
        return $this->username;
    }

    public function logout()
    {
        $this->guard()->logout();

        request()->session()->invalidate();

        $notify[] = ['success', 'You have been logged out.'];
        return redirect()->route('user.login')->withNotify($notify);
    }


    public function authenticated(Request $request, $user)
    {
        $user = auth()->user();
        $user->tv = $user->ts == 1 ? 0 : 1;
        $user->save();

        return redirect()->route('user.home');
    }

    // google authentication
    public function googleLogin()
    {
        return Socialite::driver('google')->stateless()->redirect();
    }

    public function googleLoginCallback(Request $request)
    {
        try {

            $user = Socialite::driver('google')->stateless()->user();
            $google_avatar = $user->avatar;


            $finduser = User::where('google_id', $user->id)
            ->orWhere('email', $user->email)
            ->first();
            
            
            if ($finduser) {
            
                UserProfile::where('user_id', $finduser->id)->update([
                    'image'=>$google_avatar
                ]);
                
                Auth::login($finduser);
                $this->loginLog($finduser);
                    
                return redirect()->intended('earn');
                
            } else {
                
                if(GENERAL_SETTING['registration'] == "0"){
                    $notify[] = ['error', 'Registration will open soon.'];
                    return redirect('/')->withNotify($notify);
                }
                
                
                $balance = 0;
                $referBy = session()->get('reference');
                if ($referBy) {
                    $referUser = User::where('username', $referBy)->first();
                    if ($referUser && boolval($referUser->ref_bounce)) {
                        $balance = GENERAL_SETTING['reg_ref_bounce'];
                    }
                } else {
                    $referUser = null;
                }

                $name = explode(' ', $user->name);

                $newUser = User::create([
                    'email' => $user->email,
                    'google_id' => $user->id,
                    'password' => Hash::make('123456dummy'),
                    'token_id' => generateToken(),
                    'timezone' => config('app.timezone'),
                    'ref_by' => $referUser ? $referUser->id : 0,
                    'ev' => 1,
                    'balance' => $balance,
                    'sv' => 1,
                    'firstname' => isset($name[0]) ? $name[0] : 'New',
                    'lastname' => isset($name[1]) ? $name[1] : 'User',
                    'username' => str_replace(' ', '', $user->name),
                ]);

                if ($referUser && boolval($referUser->ref_bounce) && SETTING['reg_ref_bounce'] > 0){
                    //save transactions
                    \App\Models\Transaction::store([
                        'user' => $user,
                        'amount' => SETTING['reg_ref_bounce'],
                        'from' => 'REFERRAL_BOUNCE',
                        'source_id' => $user->id,
                        'details' => 'Signup bonus',
                    ]);
                }

                UserProfile::create([
                    'user_id' => $newUser->id,
                    'image'=> $google_avatar
                ]);
        
        
                $adminNotification = new AdminNotification();
                $adminNotification->user_id = $newUser->id;
                $adminNotification->title = 'New member registered';
                $adminNotification->click_url = urlPath('moder.users.detail', $newUser->id);
                $adminNotification->save();

                Auth::login($newUser);
                $this->loginLog($newUser);

                return redirect()->intended('earn');
            }
        } catch (\Exception $e) {
             $notify[] = ['warning', "Failed using google, please use email login instead!"];
             return redirect()->route('user.login')->withNotify($notify);
        }
    }
}
